<?php

require (dirname(__FILE__).'/menutoplinks_mod.class.php');

class Blocktopmenu_mod extends Module
{
    private $_html = '';
    private $user_groups;
    private $_menuItemsArray = null;
    private $_menuItemsOptionsArray = null;
    private $_options = array(
                array('id_option' => 'auto', 'name' => 'Enable'),
                array('id_option' => 'off', 'name' => 'Disable')
            );
    private $_modes = array(
                array('value' => '', 'name' => ''),
                array('value' => 'sheet', 'name' => 'Sheet'),
                array('value' => 'custom', 'name' => 'Custom')
            ); 
    private $_defaultMode = '';
    private $_units = array(
            array('value' => 'px', 'name' => 'px'),
            array('value' => 'em', 'name' => 'em'),
            array('value' => 'percent', 'name' => '%'),
        );
    private $_layouts = array('lg', 'md', 'sm', 'xs');

    /*
     * Pattern for matching config values
     */
    private $pattern = '/^([A-Z_]*)[0-9]+/';

    /*
     * Name of the controller
     * Used to set item selected or not in top menu
     */
    private $page_name = '';

    /*
     * Spaces per depth in BO
     */
    private $spacer_size = '5';

    public function __construct()
    {
        $this->name = 'blocktopmenu_mod';
        $this->tab = 'front_office_features';
        $this->version = 1.11;
        $this->author = 'Billion Themler';

        $this->bootstrap = true;
        parent::__construct();	

        $this->displayName = $this->l('Top horizontal menu modified');
        $this->description = $this->l('Adds a new horizontal menu to the top of your e-commerce website.');
        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
        $this->_defaultMode = $this->_modes[0];
    }

    public function install()
    {
        if (!parent::install() ||
            !Configuration::updateGlobalValue('MOD_BLOCKTOPMENU_ITEMS', 'CAT3,CAT26') ||
            !Configuration::updateGlobalValue('MOD_BLOCKTOPMENU_HOME', '1') ||
            !Configuration::updateGlobalValue('MOD_BLOCKTOPMENU_MEGAMENU_ITEMS', '') ||
            !$this->registerHook('actionObjectCategoryUpdateAfter') ||
            !$this->registerHook('actionObjectCategoryDeleteAfter') ||
            !$this->registerHook('actionObjectCategoryAddAfter') ||
            !$this->registerHook('actionObjectCmsUpdateAfter') ||
            !$this->registerHook('actionObjectCmsDeleteAfter') ||
            !$this->registerHook('actionObjectCmsAddAfter') ||
            !$this->registerHook('actionObjectSupplierUpdateAfter') ||
            !$this->registerHook('actionObjectSupplierDeleteAfter') ||
            !$this->registerHook('actionObjectSupplierAddAfter') ||
            !$this->registerHook('actionObjectManufacturerUpdateAfter') ||
            !$this->registerHook('actionObjectManufacturerDeleteAfter') ||
            !$this->registerHook('actionObjectManufacturerAddAfter') ||
            !$this->registerHook('actionObjectProductUpdateAfter') ||
            !$this->registerHook('actionObjectProductDeleteAfter') ||
            !$this->registerHook('actionObjectProductAddAfter') ||
            !$this->registerHook('categoryUpdate') ||
            !$this->registerHook('actionShopDataDuplication') ||
            !$this->registerHook('displayNavigationBar') ||
            !$this->installDB())
            return false;
        return true;
    }

    public function installDb()
    {
        return (Db::getInstance()->execute('
        CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'linksmenutop` (
            `id_linksmenutop` INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
            `id_shop` INT(11) UNSIGNED NOT NULL,
            `new_window` TINYINT( 1 ) NOT NULL,
            INDEX (`id_shop`)
        ) ENGINE = '._MYSQL_ENGINE_.' CHARACTER SET utf8 COLLATE utf8_general_ci;') &&
            Db::getInstance()->execute('
             CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'linksmenutop_lang` (
            `id_linksmenutop` INT(11) UNSIGNED NOT NULL,
            `id_lang` INT(11) UNSIGNED NOT NULL,
            `id_shop` INT(11) UNSIGNED NOT NULL,
            `label` VARCHAR( 128 ) NOT NULL ,
            `link` VARCHAR( 128 ) NOT NULL ,
            INDEX ( `id_linksmenutop` , `id_lang`, `id_shop`)
        ) ENGINE = '._MYSQL_ENGINE_.' CHARACTER SET utf8 COLLATE utf8_general_ci;'));
    }

    public function uninstall()
    {
        if (!parent::uninstall() ||
            !Configuration::deleteByName('MOD_BLOCKTOPMENU_ITEMS') ||
            !Configuration::deleteByName('MOD_BLOCKTOPMENU_MEGAMENU_ITEMS') ||
            !Configuration::deleteByName('MOD_BLOCKTOPMENU_HOME') ||
            !$this->unregisterHook('displayNavigationBar') ||
            !$this->uninstallDB())
            return false;
        return true;
    }

    private function uninstallDb()
    {
        Db::getInstance()->execute('DROP TABLE `'._DB_PREFIX_.'linksmenutop`');
        Db::getInstance()->execute('DROP TABLE `'._DB_PREFIX_.'linksmenutop_lang`');
        return true;
    }

    public function getContent()
    {
        $this->context->controller->addjQueryPlugin('hoverIntent');
        
        $id_lang = (int)Context::getContext()->language->id;
        $languages = $this->context->controller->getLanguages();
        $default_language = (int)Configuration::get('PS_LANG_DEFAULT');

        $labels = Tools::getValue('label') ? array_filter(Tools::getValue('label'), 'strlen') : array();
        $links_label = Tools::getValue('link') ? array_filter(Tools::getValue('link'), 'strlen') : array();
        $spacer = str_repeat('&nbsp;', $this->spacer_size);
        $divLangName = 'link_label';

        $update_cache = false;

        if (Tools::isSubmit('submitBlocktopmenu'))
        {
            $items = Tools::getValue('items');
            if (is_array($items) && count($items) && Configuration::updateValue('MOD_BLOCKTOPMENU_ITEMS', (string)implode(',', $items)))
                $this->_html .= $this->displayConfirmation($this->l('The settings have been updated.'));
            else
                $this->_html .= $this->displayError($this->l('Unable to update settings.'));
            Configuration::updateValue('MOD_BLOCKTOPMENU_HOME', (bool)Tools::getValue('home'));
            $update_cache = true;
 
            // megamenu items must be synchronized with deleted menu items
            $megaMenuItems = $this->getSavedMegamenuItems();
            $result = array();
            if (!empty($items))
                foreach ($items as $item)
                    $result[] = $item . ',' . (array_key_exists($item, $megaMenuItems) ? (string)implode(',', $megaMenuItems[$item]) : 'auto,,0,px,,,,');
            Configuration::updateValue('MOD_BLOCKTOPMENU_MEGAMENU_ITEMS', (string)implode(';', $result));
        }
        else if (Tools::isSubmit('submitMegamenu'))
        {
            $this->saveMegamenuItems();
            $update_cache = true;
        }
        else if (Tools::isSubmit('submitBlocktopmenuLinks'))
        {
            foreach ($languages as $key => $val)
            {
                $links_label[$val['id_lang']] = Tools::getValue('link_'.(int)$val['id_lang']);
                $labels[$val['id_lang']] = Tools::getValue('label_'.(int)$val['id_lang']);
            }
            
            $count_links_label = count($links_label);
            $count_label = count($labels);
            if ($count_links_label || $count_label)
            {
                if (!$count_links_label)
                    $this->_html .= $this->displayError($this->l('Please complete the "Link" field.'));
                elseif (!$count_label)
                    $this->_html .= $this->displayError($this->l('Please add a label.'));
                elseif (!isset($labels[$default_language]))
                    $this->_html .= $this->displayError($this->l('Please add a label for your default language.'));
                else
                {
                    MenuTopLinks_mod::add($links_label, $labels,  Tools::getValue('new_window', 0), (int)Shop::getContextShopID());
                    $this->_html .= $this->displayConfirmation($this->l('The link has been added.'));
                }
            }
            $update_cache = true;
        }
        elseif (Tools::isSubmit('deletelinksmenutop'))
        {
            $id_linksmenutop = Tools::getValue('id_linksmenutop', 0);
            MenuTopLinks_mod::remove($id_linksmenutop, (int)Shop::getContextShopID());
            $items = array('MOD_BLOCKTOPMENU_ITEMS', 'MOD_BLOCKTOPMENU_MEGAMENU_ITEMS');
            foreach ($items as $item)
                Configuration::updateValue($item, str_replace(array('LNK'.$id_linksmenutop.',', 'LNK'.$id_linksmenutop), '', Configuration::get($item)));
            $this->_html .= $this->displayConfirmation($this->l('The link has been removed.'));
            $update_cache = true;
        }
        elseif (Tools::isSubmit('updatelinksmenutop'))
        {
            $id_linksmenutop = (int)Tools::getValue('id_linksmenutop', 0);
            $id_shop = (int)Shop::getContextShopID();

            if (Tools::isSubmit('updatelink'))
            {
                $link = array();
                $label = array();
                $new_window = (int)Tools::getValue('new_window', 0);
                
                foreach (Language::getLanguages(false) as $lang)
                {
                    $link[$lang['id_lang']] = Tools::getValue('link_'.(int)$lang['id_lang']);
                    $label[$lang['id_lang']] = Tools::getValue('label_'.(int)$lang['id_lang']);
                }

                MenuTopLinks_mod::update($link, $label, $new_window, (int)$id_shop, (int)$id_linksmenutop, (int)$id_linksmenutop);
                $this->_html .= $this->displayConfirmation($this->l('The link has been edited.'));
            }
            $update_cache = true;
        }
        
        if ($update_cache)
            $this->clearMenuCache();
        
        $this->_html .= $this->renderForm().$this->renderAddForm().$this->renderMegamenuForm();

        $links = MenuTopLinks_mod::gets((int)$id_lang, null, (int)Shop::getContextShopID());

        if (!count($links))
            return $this->_html;

        $this->_html .= $this->renderList();
        return $this->_html;
    }

    private function saveMegamenuItems()
    {
        $value = $this->getConfigurationValue();
        if (Configuration::updateValue('MOD_BLOCKTOPMENU_MEGAMENU_ITEMS', $value))
            $this->_html .= $this->displayConfirmation($this->l('The settings have been updated.'));
        else {
            $this->_html .= $this->displayError($this->l('Unable to update settings.'));
        }
    }

    private function getConfigurationValue()
    {
        $menu_item = $this->makeMenuOption();
        $items     = array();
        foreach ($this->_menuItemsArray as $item) 
        {
            $l = '';
            foreach($this->_layouts as $layout)
                $l .= ','.Tools::getValue($item['value'].'_LAYOUT_'.$layout, '');
            $items[] = $item['value']
                .','.Tools::getValue($item['value'], 'auto')
                .','.Tools::getValue($item['value'].'_MODE', $this->_defaultMode)
                .','.Tools::getValue($item['value'].'_CUSTOM', '0')
                .','.Tools::getValue($item['value'].'_UNIT', 'px')
                . $l;
        }
        $value = is_array($items) && count($items) ? (string)implode(';', $items) : '';
        return $value;
    }

    private function getEnabledMegamenuItems()
    {
        $items = $this->getSavedMegamenuItems();
        $result = array();
        foreach ($items as $key => $item)
        {
            if ($items[$key][0] != 'off')
                $result[] = $key;
        }
        return $result;
    }

    private function getSavedMegamenuItems()
    {
        $conf = Configuration::get('MOD_BLOCKTOPMENU_MEGAMENU_ITEMS');
        if (!$conf) // empty configuration - all menu items set in auto mode
        {
            $this->saveMegamenuItems();
            $conf = Configuration::get('MOD_BLOCKTOPMENU_MEGAMENU_ITEMS');
        }
        $result = array();
        if (strlen($conf))
        {
            $items = explode(';', Configuration::get('MOD_BLOCKTOPMENU_MEGAMENU_ITEMS'));
            foreach ($items as $item)
            {
                $i = explode(',', $item);
                $result[$i[0]] = array($i[1], isset($i[2])?$i[2]:$this->_defaultMode, isset($i[3])?$i[3]:0,  isset($i[4])?$i[4]:'px',  isset($i[5])?$i[5]:'',  isset($i[6])?$i[6]:'',  isset($i[7])?$i[7]:'',  isset($i[8])?$i[8]:'');
            }
        }
        return $result;
    }

    private function getMenuItems()
    {
        $items = Tools::getValue('items');
        if (is_array($items) && count($items))
            return $items;
        else
        {
            $conf = Configuration::get('MOD_BLOCKTOPMENU_ITEMS');
            if (strlen($conf))
                return explode(',', Configuration::get('MOD_BLOCKTOPMENU_ITEMS'));
            else
                return array();
        }
    }

    private function makeMenuOption()
    {
        $menu_item = $this->getMenuItems();

        $id_lang = (int)$this->context->language->id;
        $id_shop = (int)Shop::getContextShopID();
        $html = '<select multiple="multiple" name="items[]" id="items" style="width: 300px; height: 160px;">';
        $this->_menuItemsArray = array();
        foreach ($menu_item as $item)
        {
            if (!$item)
                continue;

            preg_match($this->pattern, $item, $values);
            $id = (int)substr($item, strlen($values[1]), strlen($item));
            $name = '';
            switch (substr($item, 0, strlen($values[1])))
            {
                case 'CAT':
                    $category = new Category((int)$id, (int)$id_lang);
                    if (Validate::isLoadedObject($category)) {
                        $value = 'CAT'.$id;
                        $name  = $category->name;
                        $html .= '<option selected="selected" value="'.$value.'">'.$name.'</option>'.PHP_EOL;
                    }
                    $this->_menuItemsArray[] = array('value' => $value, 'name' => $name);
                    break;

                case 'PRD':
                    $product = new Product((int)$id, true, (int)$id_lang);
                    if (Validate::isLoadedObject($product)) {
                        $value = 'PRD'.$id;
                        $name  = $name;
                        $html .= '<option selected="selected" value="'.$value.'">'.$name.'</option>'.PHP_EOL;
                    }
                    break;

                case 'CMS':
                    $cms = new CMS((int)$id, (int)$id_lang);
                    if (Validate::isLoadedObject($cms)) {
                        $value = 'CMS'.$id;
                        $name  = $cms->meta_title;
                        $html .= '<option selected="selected" value="'.$value.'">'.$name.'</option>'.PHP_EOL;
                    }
                    break;

                case 'CMS_CAT':
                    $category = new CMSCategory((int)$id, (int)$id_lang);
                    if (Validate::isLoadedObject($category)) {
                        $value = 'CMS_CAT'.$id;
                        $name  = $category->name;
                        $html .= '<option selected="selected" value="'.$value.'">'.$name.'</option>'.PHP_EOL;
                    }
                    $this->_menuItemsArray[] = array('value' => $value, 'name' => $name);
                    break;

                // Case to handle the option to show all Manufacturers
                case 'ALLMAN':
                    $value = 'ALLMAN0';
                    $name  = $this->l('All manufacturers');
                    $html .= '<option selected="selected" value="'.$value.'">'.$name.'</option>'.PHP_EOL;
                    break;

                case 'MAN':
                    $manufacturer = new Manufacturer((int)$id, (int)$id_lang);
                    if (Validate::isLoadedObject($manufacturer)) {
                        $value = 'MAN'.$id;
                        $name  = $manufacturer->name;
                        $html .= '<option selected="selected" value="'.$value.'">'.$name.'</option>'.PHP_EOL;
                    }
                    break;

                // Case to handle the option to show all Suppliers
                case 'ALLSUP':
                    $value = 'ALLSUP0';
                    $name  = $this->l('All suppliers');
                    $html .= '<option selected="selected" value="'.$value.'">'.$name.'</option>'.PHP_EOL;
                    break;
                    
                case 'SUP':
                    $supplier = new Supplier((int)$id, (int)$id_lang);
                    if (Validate::isLoadedObject($supplier)) {
                        $value = 'SUP'.$id;
                        $name  = $supplier->name;
                        $html .= '<option selected="selected" value="'.$value.'">'.$name.'</option>'.PHP_EOL;
                    }
                    break;

                case 'LNK':
                    $link = MenuTopLinks_mod::get((int)$id, (int)$id_lang, (int)$id_shop);
                    if (count($link))
                    {
                        if (!isset($link[0]['label']) || ($link[0]['label'] == ''))
                        {
                            $default_language = Configuration::get('PS_LANG_DEFAULT');
                            $link = MenuTopLinks_mod::get($link[0]['id_linksmenutop'], (int)$default_language, (int)Shop::getContextShopID());
                        }
                        $value = 'LNK'.$link[0]['id_linksmenutop'];
                        $name  = $link[0]['label'];
                        $html .= '<option selected="selected" value="'.$value.'">'.$name.'</option>';
                    }
                    break;
                case 'SHOP':
                    $shop = new Shop((int)$id);
                    if (Validate::isLoadedObject($shop)) {
                        $value = 'SHOP'.(int)$id;
                        $name  = $shop->name;
                        $html .= '<option selected="selected" value="'.$value.'">'.$name.'</option>'.PHP_EOL;
                    }
                    break;
            }
        }
        return $html.'</select>';
    }

    private function isMegaMenu()
    {
        return $this->_showAll === 'megamenu';
    }

    private function getColumnClass($type, $level)
    {
        $class = '';
        $indexes = array('lg'=>4, 'md'=>5, 'sm'=>6, 'xs'=>7);
        if ($level === 1 && $this->isMegaMenu()) {
            $megaMenuItems = $this->getSavedMegamenuItems();
            foreach($this->_layouts as $layout) {
                if ($this->_responsive === $layout) {
                    break;
                }
                if (!empty($megaMenuItems[$type]) && !empty($megaMenuItems[$type][$indexes[$layout]])) {
                    $class .= " col-$layout-" . (12 / $megaMenuItems[$type][$indexes[$layout]]);
                } else if (!empty($this->_columns[$layout])) {
                    $class .= " col-$layout-" . (12 / $this->_columns[$layout]);
                }
            }
        }
        return $class;
    }

    private function makeMenu()
    {
        $menu = '';
        $menu_items = $this->getMenuItems();
        $id_lang = (int)$this->context->language->id;
        $id_shop = (int)Shop::getContextShopID();
        $head = $this->_itemHead;
        $tail = $this->_itemTail;
        $megaMenuItems = $this->isMegaMenu() ? $this->getEnabledMegamenuItems() : array();
        foreach ($menu_items as $item)
        {
            if (!$item)
                continue;

            $megamenuAttrs = $this->_getMegamenuAttributes($item);
            preg_match($this->pattern, $item, $value);
            $id = (int)substr($item, strlen($value[1]), strlen($item));

            switch (substr($item, 0, strlen($value[1])))
            {
                case 'CAT':
                    $activeCategories = array();
                    $id_cat = (int)Tools::getValue('id_category', false);
                    if ($id_cat && $this->page_name == 'category')
                    {
                        $categ = new Category($id_cat, $id_lang, $id_shop);
                        $activeCategories = $categ->getParentsCategories($id_lang);
                    }
                    $useMegamenu = in_array('CAT'.$id, $megaMenuItems);
                    $this->generateCategoriesMenu($menu, Category::getNestedCategories($id, $id_lang, true, $this->user_groups), $activeCategories, false, 0, $useMegamenu, $megamenuAttrs, $item);
                    break;

                case 'PRD':
                    $selected = ($this->page_name == 'product' && (Tools::getValue('id_product') == $id)) ? ' active' : '';
                    $product = new Product((int)$id, true, (int)$id_lang);
                    if (!is_null($product->id))
                        $menu .= $head . '<li class="'. $this->_itemClass . $selected . '" '.$this->_itemAttr.'><a href="'.Tools::HtmlEntitiesUTF8($product->getLink()).'" title="'.$product->name.'"><span>'.$product->name.'</span></a></li>'.$tail.PHP_EOL;
                    break;

                case 'CMS':
                    $selected = ($this->page_name == 'cms' && (Tools::getValue('id_cms') == $id)) ? ' class="active"' : '';
                    $cms = CMS::getLinks((int)$id_lang, array($id));
                    if (count($cms))
                        $menu .= $head . '<li class="'. $this->_itemClass . '" '.$this->_itemAttr.'><a href="'.Tools::HtmlEntitiesUTF8($cms[0]['link']).'" title="'.Tools::safeOutput($cms[0]['meta_title']).'"'.$selected .'><span>'.Tools::safeOutput($cms[0]['meta_title']).'</span></a></li>'.$tail.PHP_EOL;
                    break;

                case 'CMS_CAT':
                    $selected = ($this->page_name == 'cms' && (Tools::getValue('id_cms_category') == $id)) ? ' class="active"' : '';
                    $category = new CMSCategory((int)$id, (int)$id_lang);
                    $useMegamenu = in_array('CMS_CAT'.$id, $megaMenuItems);
                    if (count($category))
                    {
                        $submenu = '';
                        $depth = $this->getCMSMenuItems($submenu, $category->id, 1, false, $useMegamenu, $item);
                        $menu .= $head . '<li class="' . $this->_getItemFullClass($depth > 1 ? $useMegamenu : false) . '" '.$this->_itemAttr.'><a href="'.Tools::HtmlEntitiesUTF8($category->getLink()).'" title="'.$category->name.'"'.$selected .$megamenuAttrs.'><span>'.$category->name.'</span></a>';
                        $menu .= $submenu . '</li>' . $tail.PHP_EOL;
                    }
                    break;

                // Case to handle the option to show all Manufacturers
                case 'ALLMAN':
                    $selected = ($this->page_name == 'manufacturer') ? ' class="active"' : '';
                    $link = new Link;
                    $useMegamenu = in_array('ALLMAN'.$id, $megaMenuItems);
                    $menu .= $head . '<li class="'. $this->_getItemFullClass(false) . '" '.$this->_itemAttr.'><a href="'.$link->getPageLink('manufacturer').'" title="'.$this->l('All manufacturers').'"'. $selected.'><span>'.$this->l('All manufacturers').'</span></a>';

                    $manufacturers = Manufacturer::getManufacturers();
                    $sub_menu_active = '';
                    $sub_menu = '';
                    foreach ($manufacturers as $key => $manufacturer) {
                        $id_man = (int)$manufacturer['id_manufacturer'];
                        $active = (Tools::getValue('id_manufacturer') == $id_man) ? ' class="active"' : '';
                        $sub_menu_active = empty($active) ? $sub_menu_active : ' active';
                        $class = $this->_subItemClass . ($useMegamenu ? $this->_megaCategoryItem : '');
                        $sub_menu .= $this->_subItemHead . '<li class="'. $class . '" '.$this->_subItemAttr.'><a href="'.$link->getManufacturerLink($id_man, $manufacturer['link_rewrite']).'" title="'.Tools::safeOutput($manufacturer['name']).'"' . $active . '><span>'.Tools::safeOutput($manufacturer['name']).'</span></a></li>'. $this->_subItemTail. PHP_EOL;
                    }

                    
                    $menu .= $this->_subMenuHead . '<ul class="'.$this->_subMenuClass. $sub_menu_active. '" '.$this->_subMenuAttr.'>'.$sub_menu . '</ul>' . $this->_subMenuTail;
                    $menu .= '</li>'.$tail;
                    break;

                case 'MAN':
                    $selected = ($this->page_name == 'manufacturer' && (Tools::getValue('id_manufacturer') == $id)) ? ' class="active"' : '';
                    $manufacturer = new Manufacturer((int)$id, (int)$id_lang);
                    if (!is_null($manufacturer->id))
                    {
                        if (intval(Configuration::get('PS_REWRITING_SETTINGS')))
                            $manufacturer->link_rewrite = Tools::link_rewrite($manufacturer->name);
                        else
                            $manufacturer->link_rewrite = 0;
                        $link = new Link;
                        $menu .= $head . '<li class="'. $this->_itemClass . '" '.$this->_itemAttr.'><a href="'.Tools::HtmlEntitiesUTF8($link->getManufacturerLink((int)$id, $manufacturer->link_rewrite)).'" title="'.Tools::safeOutput($manufacturer->name).'"'. $selected .'><span>'.Tools::safeOutput($manufacturer->name).'</span></a></li>'.$tail.PHP_EOL;
                    }
                    break;

                // Case to handle the option to show all Suppliers
                case 'ALLSUP':
                    $selected = ($this->page_name == 'supplier') ? ' class="active"' : '';
                    $link = new Link;
                    $useMegamenu = in_array('ALLSUP'.$id, $megaMenuItems);
                    $menu .= $head . '<li class="'. $this->_getItemFullClass(false) . '" '.$this->_itemAttr.'><a href="'.$link->getPageLink('supplier').'" title="'.$this->l('All suppliers').'"' . $selected .'><span>'.$this->l('All suppliers').'</span></a>';
                    
                    $suppliers = Supplier::getSuppliers();
                    $sub_menu_active = '';
                    $sub_menu = '';
                    foreach ($suppliers as $key => $supplier) {
                        $id_sup = (int)$supplier['id_supplier'];
                        $active = (Tools::getValue('id_supplier') == $id_sup) ? ' class="active"' : '';
                        $sub_menu_active = empty($active) ? $sub_menu_active : ' active';
                        $class = $this->_subItemClass . ($useMegamenu ? $this->_megaCategoryItem : '');
                        $sub_menu .= $this->_subItemHead . '<li class="'. $class . '" '.$this->_subItemAttr.'><a href="'.$link->getSupplierLink($id_sup, $supplier['link_rewrite']).'" title="'.Tools::safeOutput($supplier['name']).'"' . $active . '><span>'.Tools::safeOutput($supplier['name']).'</span></a></li>'. $this->_subItemTail. PHP_EOL;
                    }

                    $menu .= $this->_subMenuHead . '<ul class="'.$this->_subMenuClass . $sub_menu_active.'" '.$this->_subMenuAttr.'>'.$sub_menu . '</ul>' . $this->_subMenuTail;
                    $menu .= '</li>'.$tail;
                    break;

                case 'SUP':
                    $selected = ($this->page_name == 'supplier' && (Tools::getValue('id_supplier') == $id)) ? ' class="active"' : '';
                    $supplier = new Supplier((int)$id, (int)$id_lang);
                    if (!is_null($supplier->id))
                    {
                        $link = new Link;
                        $menu .= $head . '<li class="'. $this->_itemClass . '" '.$this->_itemAttr.'><a href="'.Tools::HtmlEntitiesUTF8($link->getSupplierLink((int)$id, $supplier->link_rewrite)).'" title="'.$supplier->name.'"' . $selected .'><span>'.$supplier->name.'</span></a></li>'.$tail.PHP_EOL;
                    }
                    break;

                case 'SHOP':
                    $selected = ($this->page_name == 'index' && ($this->context->shop->id == $id)) ? ' class="active"' : '';
                    $shop = new Shop((int)$id);
                    if (Validate::isLoadedObject($shop))
                    {
                        $link = new Link;
                        $menu .= $head . '<li class="'. $this->_itemClass . '" '.$this->_itemAttr.'><a href="'.Tools::HtmlEntitiesUTF8($shop->getBaseURL()).'" title="'.$shop->name.'"' . $selected . '><span>'.$shop->name.'</span></a></li>'.$tail.PHP_EOL;
                    }
                    break;
                case 'LNK':
                    $link = MenuTopLinks_mod::get((int)$id, (int)$id_lang, (int)$id_shop);
                    if (count($link))
                    {
                        if (!isset($link[0]['label']) || ($link[0]['label'] == ''))
                        {
                            $default_language = Configuration::get('PS_LANG_DEFAULT');
                            $link = MenuTopLinks_mod::get($link[0]['id_linksmenutop'], $default_language, (int)Shop::getContextShopID());
                        }
                        $menu .= $head . '<li class="'. $this->_itemClass . '" '.$this->_itemAttr.'><a href="'.Tools::HtmlEntitiesUTF8($link[0]['link']).'"'.(($link[0]['new_window']) ? ' onclick="return !window.open(this.href);"': '').' title="'.Tools::safeOutput($link[0]['label']).'"><span>'.Tools::safeOutput($link[0]['label']).'</span></a></li>'.$tail.PHP_EOL;
                    }
                    break;
            }
        }

        return $menu;
    }

    private function _getItemFullClass($useMegamenu = false)
    {
        return ' bd-submenu-icon-only '. $this->_itemClass . ($useMegamenu ? $this->_megaMenu : '');
    }

    private function _getMegamenuAttributes($item)
    {
        $megamenu = $this->getSavedMegamenuItems();
        if (array_key_exists($item, $megamenu))
        {
            $mode  = $this->_popup_mode;
            $width = $this->_popup_custom_width;
            if (!empty($megamenu[$item][1]))
            {
                $mode  = $megamenu[$item][1];
                $width = $megamenu[$item][2]; //  . $megamenu[$item][3]
            }
            return ' data-mega-width="' . $mode .'"'. ($mode === 'custom' ? ' data-mega-width-value="'. $width .'"' : '');
        }
        return '';
    }

    private function getActiveCMSMenuItemPath($id, $id_lang = false)
    {
        $id_lang = $id_lang ? (int)$id_lang : (int)Context::getContext()->language->id;

		$categories = null;
		$id_current = $id;
		while (true)
		{
			$query = '
				SELECT c.*, cl.*
				FROM `'._DB_PREFIX_.'cms_category` c
				LEFT JOIN `'._DB_PREFIX_.'cms_category_lang` cl ON (c.`id_cms_category` = cl.`id_cms_category` AND `id_lang` = '.(int)$id_lang.')
				WHERE c.`id_cms_category` = '.(int)$id_current.' AND c.`id_parent` != 0
			';
			$result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query);

			$categories[] = isset($result[0]) ? $result[0] : null;
			if (!$result || $result[0]['id_parent'] == 1)
				return $categories;
			$id_current = $result[0]['id_parent'];
		}
        
        return $categories;
    }
    
    private function generateCategoriesOption($categories, $items_to_skip = null)
    {
        $html = '';

        foreach ($categories as $key => $category)
        {
            if (isset($items_to_skip) && !in_array('CAT'.(int)$category['id_category'], $items_to_skip))
            {
                $shop = (object) Shop::getShop((int)$category['id_shop']);
                $html .= '<option value="CAT'.(int)$category['id_category'].'">'
                    .str_repeat('&nbsp;', $this->spacer_size * (int)$category['level_depth']).$category['name'].' ('.$shop->name.')</option>';
            }

            if (isset($category['children']) && !empty($category['children']))
                $html .= $this->generateCategoriesOption($category['children'], $items_to_skip);

        }

        return $html;
    }

    private function generateCategoriesMenu(&$html, $categories, $activeCategories = array(), $is_child_item = false, $level = 0, $useMegamenu = false, $megamenuAttrs = '', $item)
    {
        $result = $level;
        $class = $is_child_item ? $this->_subItemClass : $this->_itemClass;
        $attr = $is_child_item ? $this->_subItemAttr : $this->_itemAttr;
        $head = $is_child_item ? $this->_subItemHead : $this->_itemHead;
        $tail = $is_child_item ? $this->_subItemTail : $this->_itemTail;
        
        foreach ($categories as $key => $category)
        {
            $currentId = (int)$category['id_category'];
            if ($category['level_depth'] > 1)
            {
                $cat = new Category($currentId);
                $link = Tools::HtmlEntitiesUTF8($cat->getLink());
            }
            else
                $link = $this->context->link->getPageLink('index');

            $selected = '';
            foreach ($activeCategories as $activeCategory)
            {
                if ($currentId === (int)$activeCategory['id_category']) 
                    $selected = ' active';
            }

            $megaMenuItem = $this->getMegamenuItemClass($level, $useMegamenu);

            $submenu = '';
            if (isset($category['children']) && !empty($category['children']) && ($this->_showAll !== 'one level' || $this->_showAllResponsive !== 'one level'))
            {
                $popup    = $this->getSubMenuTails($useMegamenu, $level);
                $submenu .= $popup['head'];
                $result   = max($result, $this->generateCategoriesMenu($submenu, $category['children'], $activeCategories, true, $level + 1, $useMegamenu, $megamenuAttrs, $item));
                $submenu .= $popup['tail'];
            }

            $linkAttr = '';
            if (!$is_child_item && $result > 0 && isset($category['children']) && !empty($category['children'])) {
                $class .= $this->_getItemFullClass($useMegamenu);
                $linkAttr = $megamenuAttrs;
            }

            $popup = $this->getItemTails($useMegamenu, $level, $class . $megaMenuItem, $attr, $item);
            $html .= $head . $popup['head'];
            $html .= '<a href="'.$link.'" title="'.$category['name'].'"' . (empty($selected) ? '' : 'class="'.$selected.'"').$linkAttr.'><span>'.$category['name'].'</span></a>';
            $html .= $submenu. $popup['tail'] . $tail;
        }

        return $result;
    }

    private function getSubMenuTails($useMegaMenu, $level)
    {
        $head = $this->_subMenuHead . $this->_subMenuUlHead;
        $tail = $this->_subMenuUlTail . $this->_subMenuTail;
        if ($useMegaMenu && $level < 1) {
            $head = $this->_subMenuHead . $this->_subMenuMegaHead;
            $tail = $this->_subMenuMegaTail . $this->_subMenuTail;
        } 
        $head = str_replace('FIRST_LEVEL_POPUP_CLASS', $useMegaMenu && $level === 1 ? 'bd-megamenu-popup' : '',  $head);
        return array('head' => $head, 'tail' => $tail);
    }

    private function getItemTails($useMegaMenu, $level, $class, $attr, $item)
    {
        $head = $this->_subItemHead . $this->_subItemLiHead;
        $tail = $this->_subItemLiTail . $this->_subItemTail;
        if ($level < 1) // TODO: refactor and remove separate class vars
        {
            $head = '<li class="'. $class .'" '.$attr.'>';
            $tail = '</li>';
        }
        $replace = ' bd-sub-item ';
        if ($useMegaMenu) 
        {
            $replace .= $this->getColumnClass($item, $level);
            if ($level == 1)
            {
                $head = $this->_subItemHead . $this->_subItemMegaHead;
                $tail = $this->_subItemMegaTail . $this->_subItemTail;
                $replace .= $this->_subItemMegaClass;
            } 
            else
                $replace .= $this->_subSubItemMegaClass;
        }
        $head = str_replace('CLASS', $replace, $head);
        return array('head' => $head, 'tail' => $tail);
    }
    
    private function getCmsParentCategory($id_cms)
	{
		$sql = '
		SELECT id_cms_category
		FROM `'._DB_PREFIX_.'cms`
		WHERE `id_cms` = '.(int)$id_cms;

		return (int)(Db::getInstance()->getValue($sql));
	}

    private function getMegamenuItemClass($level, $useMegamenu = false)
    {
        if ($useMegamenu)
        {
            if ($level === 1)
                return $this->_megaCategoryItem;
            if ($level === 2)
                return $this->_megaSubcategoryItem;
        }
        return '';
    }

    private function getCMSMenuItems(&$menu, $parent, $depth = 1, $id_lang = false, $useMegamenu = false, $item)
    {
        $result = $depth;
        $category_id = (int)Tools::getValue('id_cms_category', false);
        $page_id = (int)Tools::getValue('id_cms', false);
        $is_category = $this->page_name == 'cms' && $category_id;
        $is_page = $this->page_name == 'cms' && $page_id;
        $id_lang = $id_lang ? (int)$id_lang : (int)Context::getContext()->language->id;
        
        $activeCategories = array();
        if ($is_page) 
        {
            $parent_category = $this->getCmsParentCategory($page_id);
            $activeCategories = $this->getActiveCMSMenuItemPath($parent_category, $id_lang);
        }
        else if ($is_category)
        {
            $activeCategories = $this->getActiveCMSMenuItemPath($category_id, $id_lang);
        }

        $categories = $this->getCMSCategories(false, (int)$parent, (int)$id_lang);
        $pages = $this->getCMSPages((int)$parent);

        if ((count($categories) || count($pages)) && ($this->_showAll !== 'one level' || $this->_showAllResponsive !== 'one level'))
        {
            $pageMenu = '';
            foreach ($pages as $page)
            {
                $cms = new CMS($page['id_cms'], (int)$id_lang);
                $links = $cms->getLinks((int)$id_lang, array((int)$cms->id));

                $selected = ($this->page_name == 'cms' && ((int)Tools::getValue('id_cms') == $page['id_cms'])) ? ' class="active"' : '';

                $popup = $this->getItemTails($useMegamenu, $depth, $this->_subItemClass . $this->getMegamenuItemClass($depth, $useMegamenu), $this->_subItemAttr, $item);
                $pageMenu .= $popup['head'];
                $pageMenu .= '<a href="'.$links[0]['link'].'"' . $selected . '><span>'.$cms->meta_title.'</span></a>';
                $pageMenu .= $popup['tail'];
            }

            $catMenu = '';
            foreach ($categories as $category)
            {
                $currentId = (int)$category['id_cms_category'];
                $catSelect = '';
                foreach ($activeCategories as $activeCategory)
                {
                    if ($currentId === (int)$activeCategory['id_cms_category']) 
                        $catSelect = ' class="active"';
                }
                $cat = new CMSCategory((int)$category['id_cms_category'], (int)$id_lang);
                
                $popup = $this->getItemTails($useMegamenu, $depth, $this->_subItemClass, $this->_subItemAttr, $item);
                $catMenu .= $popup['head'];
                $catMenu .= '<a href="'.Tools::HtmlEntitiesUTF8($cat->getLink()).'"' . $catSelect. '><span>'.$category['name'].'</span></a>';
                $result = max($result, $this->getCMSMenuItems($catMenu, $category['id_cms_category'], (int)$depth + 1, $id_lang, $useMegamenu, $item));
                $catMenu .= $popup['tail'];
            }

            $popup = $this->getSubMenuTails($useMegamenu, $depth-1);
            $menu .= $popup['head'];
            $menu .= $catMenu;
            $menu .= $pageMenu;
            $menu .= $popup['tail'];
        }
        return $result;
    }

    protected function getCMSOptions($parent = 0, $depth = 1, $id_lang = false, $items_to_skip = null, $id_shop = false)
    {
        $html = '';
        $id_lang = $id_lang ? (int)$id_lang : (int)Context::getContext()->language->id;
        $id_shop = ($id_shop !== false) ? $id_shop : Context::getContext()->shop->id;
        $categories = $this->getCMSCategories(false, (int)$parent, (int)$id_lang, (int)$id_shop);
        $pages = $this->getCMSPages((int)$parent, false, (int)$id_lang);

        $spacer = str_repeat('&nbsp;', $this->spacer_size * (int)$depth);

        foreach ($categories as $category)
        {
            if (isset($items_to_skip) && !in_array('CMS_CAT'.$category['id_cms_category'], $items_to_skip))
                $html .= '<option value="CMS_CAT'.$category['id_cms_category'].'" style="font-weight: bold;">'.$spacer.$category['name'].'</option>';
            $html .= $this->getCMSOptions($category['id_cms_category'], (int)$depth + 1, (int)$id_lang, $items_to_skip);
        }

        foreach ($pages as $page)
            if (isset($items_to_skip) && !in_array('CMS'.$page['id_cms'], $items_to_skip))
                $html .= '<option value="CMS'.$page['id_cms'].'">'.$spacer.$page['meta_title'].'</option>';

        return $html;
    }
    
    protected function getCacheId($name = null)
    {
        parent::getCacheId($name);
        $page_name = in_array($this->page_name, array('category', 'supplier', 'manufacturer', 'cms', 'product')) ? $this->page_name : 'index';
        return 'blocktopmenu_mod|'.(int)Tools::usingSecureMode().'|'.$page_name.'|'.(int)$this->context->shop->id.'|'.implode(', ',$this->user_groups).'|'.(int)$this->context->language->id.'|'.(int)Tools::getValue('id_category').'|'.(int)Tools::getValue('id_manufacturer').'|'.(int)Tools::getValue('id_supplier').'|'.(int)Tools::getValue('id_cms').'|'.(int)Tools::getValue('id_product');
    }

    public function hookDisplayNavigationBar($args)
    {
        $params = isset($args['menu']) && is_array($args['menu']) ? $args['menu'] : $args;
        foreach ($params as $key => $value)
        {
            $name = '_' . str_replace(array('submenu', 'subitem'), array('subMenu', 'subItem'), $key);
            $this->$name = $value;
        }

        $this->user_groups =  ($this->context->customer->isLogged() ? $this->context->customer->getGroups() : array(Configuration::get('PS_UNIDENTIFIED_GROUP')));
        $this->page_name = Dispatcher::getInstance()->getController();
        if (!$this->isCached('blocktopmenu_mod.tpl', $this->getCacheId()))
        {
            $menu = $this->makeMenu();
            $this->smarty->assign('MENU_HOME', Configuration::get('MOD_BLOCKTOPMENU_HOME'));
            $this->smarty->assign('MENU', $menu);
            $this->smarty->assign('this_path', $this->_path);
            $this->smarty->assign('subMenuClass', $this->_subMenuClass);
            $this->smarty->assign('subMenuAttr', $this->_subMenuAttr);
            $this->smarty->assign('menuItemClass', $this->_itemClass);
            $this->smarty->assign('menuItemAttr', $this->_itemAttr);
            $this->smarty->assign('submenuItemClassName', $this->_subItemClass);
            $this->smarty->assign('submenuItemAttr', $this->_subItemAttr);
        }

        $html = $this->display(__FILE__, 'blocktopmenu_mod.tpl', $this->getCacheId());
        return $html;
    }

    protected function getCMSCategories($recursive = false, $parent = 1, $id_lang = false, $id_shop = false)
    {
        $id_lang = $id_lang ? (int)$id_lang : (int)Context::getContext()->language->id;
        $id_shop = ($id_shop !== false) ? $id_shop : Context::getContext()->shop->id;
        $join_shop = '';
        $where_shop = '';

        if (Tools::version_compare(_PS_VERSION_, '1.6.0.12', '>=') == true) {
            $join_shop = ' INNER JOIN `'._DB_PREFIX_.'cms_category_shop` cs
            ON (bcp.`id_cms_category` = cs.`id_cms_category`)';
            $where_shop = ' AND cs.`id_shop` = '.(int)$id_shop.' AND cl.`id_shop` = '.(int)$id_shop;
        }

        if ($recursive === false) {
            $sql = 'SELECT bcp.`id_cms_category`, bcp.`id_parent`, bcp.`level_depth`, bcp.`active`, bcp.`position`, cl.`name`, cl.`link_rewrite`
				FROM `'._DB_PREFIX_.'cms_category` bcp'.
                $join_shop.'
				INNER JOIN `'._DB_PREFIX_.'cms_category_lang` cl
				ON (bcp.`id_cms_category` = cl.`id_cms_category`)
				WHERE cl.`id_lang` = '.(int)$id_lang.'
				AND bcp.`id_parent` = '.(int)$parent.
                $where_shop;

            return Db::getInstance()->executeS($sql);
        } else {
            $sql = 'SELECT bcp.`id_cms_category`, bcp.`id_parent`, bcp.`level_depth`, bcp.`active`, bcp.`position`, cl.`name`, cl.`link_rewrite`
				FROM `'._DB_PREFIX_.'cms_category` bcp'.
                $join_shop.'
				INNER JOIN `'._DB_PREFIX_.'cms_category_lang` cl
				ON (bcp.`id_cms_category` = cl.`id_cms_category`)
				WHERE cl.`id_lang` = '.(int)$id_lang.'
				AND bcp.`id_parent` = '.(int)$parent.
                $where_shop;

            $results = Db::getInstance()->executeS($sql);
            foreach ($results as $result) {
                $sub_categories = $this->getCMSCategories(true, $result['id_cms_category'], (int)$id_lang);
                if ($sub_categories && count($sub_categories) > 0) {
                    $result['sub_categories'] = $sub_categories;
                }
                $categories[] = $result;
            }

            return isset($categories) ? $categories : false;
        }

    }

    private function getCMSPages($id_cms_category, $id_shop = false, $id_lang = false)
    {
        $id_shop = ($id_shop !== false) ? (int)$id_shop : (int)Context::getContext()->shop->id;
        $id_lang = $id_lang ? (int)$id_lang : (int)Context::getContext()->language->id;

        $where_shop = '';
        if (Tools::version_compare(_PS_VERSION_, '1.6.0.12', '>=') == true) {
            $where_shop = ' AND cl.`id_shop` = '.(int)$id_shop;
        }

        $sql = 'SELECT c.`id_cms`, cl.`meta_title`, cl.`link_rewrite`
			FROM `'._DB_PREFIX_.'cms` c
			INNER JOIN `'._DB_PREFIX_.'cms_shop` cs
			ON (c.`id_cms` = cs.`id_cms`)
			INNER JOIN `'._DB_PREFIX_.'cms_lang` cl
			ON (c.`id_cms` = cl.`id_cms`)
			WHERE c.`id_cms_category` = '.(int)$id_cms_category.'
			AND cs.`id_shop` = '.(int)$id_shop.'
			AND cl.`id_lang` = '.(int)$id_lang.
            $where_shop.'
			AND c.`active` = 1
			ORDER BY `position`';

        return Db::getInstance()->executeS($sql);
    }

    public function hookActionObjectCategoryAddAfter($params)
    {
        $this->clearMenuCache();
    }

    public function hookActionObjectCategoryUpdateAfter($params)
    {
        $this->clearMenuCache();
    }
    
    public function hookActionObjectCategoryDeleteAfter($params)
    {
        $this->clearMenuCache();
    }
    
    public function hookActionObjectCmsUpdateAfter($params)
    {
        $this->clearMenuCache();
    }
    
    public function hookActionObjectCmsDeleteAfter($params)
    {
        $this->clearMenuCache();
    }
    
    public function hookActionObjectCmsAddAfter($params)
    {
        $this->clearMenuCache();
    }
    
    public function hookActionObjectSupplierUpdateAfter($params)
    {
        $this->clearMenuCache();
    }
    
    public function hookActionObjectSupplierDeleteAfter($params)
    {
        $this->clearMenuCache();
    }
    
    public function hookActionObjectSupplierAddAfter($params)
    {
        $this->clearMenuCache();
    }

    public function hookActionObjectManufacturerUpdateAfter($params)
    {
        $this->clearMenuCache();
    }
    
    public function hookActionObjectManufacturerDeleteAfter($params)
    {
        $this->clearMenuCache();
    }
    
    public function hookActionObjectManufacturerAddAfter($params)
    {
        $this->clearMenuCache();
    }
    
    public function hookActionObjectProductUpdateAfter($params)
    {
        $this->clearMenuCache();
    }
    
    public function hookActionObjectProductDeleteAfter($params)
    {
        $this->clearMenuCache();
    }
    
    public function hookActionObjectProductAddAfter($params)
    {
        $this->clearMenuCache();
    }
    
    public function hookCategoryUpdate($params)
    {
        $this->clearMenuCache();
    }
    
    private function clearMenuCache()
    {
        $this->_clearCache('blocktopmenu_mod.tpl');
    }
    
    public function hookActionShopDataDuplication($params)
    {
        $linksmenutop = Db::getInstance()->executeS('
            SELECT *
            FROM '._DB_PREFIX_.'linksmenutop 
            WHERE id_shop = '.(int)$params['old_id_shop']
            );

        foreach($linksmenutop as $id => $link)
        {
            Db::getInstance()->execute('
                INSERT IGNORE INTO '._DB_PREFIX_.'linksmenutop (id_linksmenutop, id_shop, new_window) 
                VALUES (null, '.(int)$params['new_id_shop'].', '.(int)$link['new_window'].')');
            
            $linksmenutop[$id]['new_id_linksmenutop'] = Db::getInstance()->Insert_ID();
        }
        
        foreach($linksmenutop as $id => $link)
        {
            $lang = Db::getInstance()->executeS('
                    SELECT id_lang, '.(int)$params['new_id_shop'].', label, link 
                    FROM '._DB_PREFIX_.'linksmenutop_lang 
                    WHERE id_linksmenutop = '.(int)$link['id_linksmenutop'].' AND id_shop = '.(int)$params['old_id_shop']);
            
            foreach($lang as $l)
                Db::getInstance()->execute('
                    INSERT IGNORE INTO '._DB_PREFIX_.'linksmenutop_lang (id_linksmenutop, id_lang, id_shop, label, link) 
                    VALUES ('.(int)$link['new_id_linksmenutop'].', '.(int)$l['id_lang'].', '.(int)$params['new_id_shop'].', '.(int)$l['label'].', '.(int)$l['link'].' )');
        }
    }
    
    public function renderForm()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Menu Top Link'),
                    'icon' => 'icon-link'
                ),
                'input' => array(
                    array(
                        'type' => 'link_choice',
                        'label' => '',
                        'name' => 'link',
                        'lang' => true,
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Home Item'),
                        'name' => 'home',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    )		
                ),
                'submit' => array(
                    'name' => 'submitBlocktopmenu',
                    'title' => $this->l('Save')
                )
            ),
        );
        
        if (Shop::isFeatureActive())
            $fields_form['form']['description'] = $this->l('The modifications will be applied to').' '.(Shop::getContext() == Shop::CONTEXT_SHOP ? $this->l('shop').' '.$this->context->shop->name : $this->l('all shops'));
        
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table =  $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->module = $this;
        $helper->identifier = $this->identifier;		
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'choices' => $this->renderChoicesSelect(),
            'selected_links' => $this->makeMenuOption(),
        );
        return $helper->generateForm(array($fields_form));
    }
    
    public function renderAddForm()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => (Tools::getIsset('updatelinksmenutop') && !Tools::getValue('updatelinksmenutop')) ? $this->l('Update link') : $this->l('Add a new link'),
                    'icon' => 'icon-link'
                ),
                'input' => array(
                    array(
                        'type' => 'text',
                        'label' => $this->l('Label'),
                        'name' => 'label',
                        'lang' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Link'),
                        'name' => 'link',
                        'lang' => true,
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('New window'),
                        'name' => 'new_window',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    )		
                ),
                'submit' => array(
                    'name' => 'submitBlocktopmenuLinks',
                    'title' => $this->l('Add')
                )
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table =  $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->identifier = $this->identifier;
        $helper->fields_value = $this->getAddLinkFieldsValues();

        if (Tools::getIsset('updatelinksmenutop') && !Tools::getValue('updatelinksmenutop'))
            $fields_form['form']['submit'] = array(
                'name' => 'updatelinksmenutop',
                'title' => $this->l('Update')
            );

        if (Tools::isSubmit('updatelinksmenutop'))
        {			
            $fields_form['form']['input'][] = array('type' => 'hidden', 'name' => 'updatelink');
            $fields_form['form']['input'][] = array('type' => 'hidden', 'name' => 'id_linksmenutop');			
            $helper->fields_value['updatelink'] = '';
        }
            
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->languages =$this->context->controller->getLanguages();
        $helper->default_form_language = (int)$this->context->language->id;

        return $helper->generateForm(array($fields_form));
    }

    public function renderMegamenuForm()
    {
        $menu_item = $this->makeMenuOption();
        $input = array();
        $fields_value = array();
        $fields_param = array();
        $savedItems = $this->getSavedMegamenuItems();
        foreach ($this->_menuItemsArray as $item) 
        {
            $input[] = array('type' => 'select', 'label' => $item['name'], 'name' => $item['value'], 'options' => array('query' => $this->_options, 'id' => 'id_option', 'name' => 'name'), 'class' => 'fixed-width-sm');
            if (array_key_exists($item['value'], $savedItems))
            {
                $fields_value[$item['value']] = $savedItems[$item['value']][0];
                $fields_param[$item['value']] = $savedItems[$item['value']];
            }
        }
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Megamenu settings'),
                    'icon' => 'icon-link'
                ),
                'input' => $input,
                'submit' => array(
                    'name' => 'submitMegamenu',
                    'title' => $this->l('Save')
                )
            ),
        );

        if (Shop::isFeatureActive())
            $fields_form['form']['description'] = $this->l('The modifications will be applied to').' '.(Shop::getContext() == Shop::CONTEXT_SHOP ? $this->l('shop').' '.$this->context->shop->name : $this->l('all shops'));
        
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table =  $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->module = $this;
        $helper->identifier = $this->identifier;		
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $fields_value,
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'modes' => $this->_modes,
            'params' => $fields_param,
            'units' => $this->_units,
            'layouts' => $this->_layouts,
        );
        return $helper->generateForm(array($fields_form));
    }

    public function renderChoicesSelect()
    {
        $spacer = str_repeat('&nbsp;', $this->spacer_size);
        $items = $this->getMenuItems();
        
        $html = '<select multiple="multiple" id="availableItems" style="width: 300px; height: 160px;">';
        $html .= '<optgroup label="'.$this->l('CMS').'">';
        $html .= $this->getCMSOptions(0, 1, $this->context->language->id, $items);
        $html .= '</optgroup>';

        // BEGIN SUPPLIER
        $html .= '<optgroup label="'.$this->l('Supplier').'">';
        // Option to show all Suppliers
        $html .= '<option value="ALLSUP0">'.$this->l('All suppliers').'</option>';
        $suppliers = Supplier::getSuppliers(false, $this->context->language->id);
        foreach ($suppliers as $supplier)
            if (!in_array('SUP'.$supplier['id_supplier'], $items))
                $html .= '<option value="SUP'.$supplier['id_supplier'].'">'.$spacer.$supplier['name'].'</option>';
        $html .= '</optgroup>';

        // BEGIN Manufacturer
        $html .= '<optgroup label="'.$this->l('Manufacturer').'">';
        // Option to show all Manufacturers
        $html .= '<option value="ALLMAN0">'.$this->l('All manufacturers').'</option>';
        $manufacturers = Manufacturer::getManufacturers(false, $this->context->language->id);
        foreach ($manufacturers as $manufacturer)
            if (!in_array('MAN'.$manufacturer['id_manufacturer'], $items))
                $html .= '<option value="MAN'.$manufacturer['id_manufacturer'].'">'.$spacer.$manufacturer['name'].'</option>';
        $html .= '</optgroup>';

        // BEGIN Categories
        $shop = new Shop((int)Shop::getContextShopID());
        $html .= '<optgroup label="'.$this->l('Categories').'">';	
        $html .= $this->generateCategoriesOption(
            Category::getNestedCategories(null, (int)$this->context->language->id, true), $items);
        $html .= '</optgroup>';
        
        // BEGIN Shops
        if (Shop::isFeatureActive())
        {
            $html .= '<optgroup label="'.$this->l('Shops').'">';
            $shops = Shop::getShopsCollection();
            foreach ($shops as $shop)
            {
                if (!$shop->setUrl() && !$shop->getBaseURL())
                    continue;

                if (!in_array('SHOP'.(int)$shop->id, $items))
                    $html .= '<option value="SHOP'.(int)$shop->id.'">'.$spacer.$shop->name.'</option>';
            }	
            $html .= '</optgroup>';
        }
        
        // BEGIN Products
        $html .= '<optgroup label="'.$this->l('Products').'">';
        $html .= '<option value="PRODUCT" style="font-style:italic">'.$spacer.$this->l('Choose product ID').'</option>';
        $html .= '</optgroup>';

        // BEGIN Menu Top Links
        $html .= '<optgroup label="'.$this->l('Menu Top Links').'">';
        $links = MenuTopLinks_mod::gets($this->context->language->id, null, (int)Shop::getContextShopID());
        foreach ($links as $link)
        {
            if ($link['label'] == '')
            {
                $default_language = Configuration::get('PS_LANG_DEFAULT');
                $link = MenuTopLinks_mod::get($link['id_linksmenutop'], $default_language, (int)Shop::getContextShopID());
                if (!in_array('LNK'.(int)$link[0]['id_linksmenutop'], $items))
                    $html .= '<option value="LNK'.(int)$link[0]['id_linksmenutop'].'">'.$spacer.$link[0]['label'].'</option>';
            }
            elseif (!in_array('LNK'.(int)$link['id_linksmenutop'], $items))
                $html .= '<option value="LNK'.(int)$link['id_linksmenutop'].'">'.$spacer.$link['label'].'</option>';
        }
        $html .= '</optgroup>';
        $html .= '</select>';
        return $html;
    }
    
    public function getConfigFieldsValues()
    {
        return array(
            'home' => Tools::getValue('home', Configuration::get('MOD_BLOCKTOPMENU_HOME'))
        );
    }

    public function getAddLinkFieldsValues()
    {
        $links_label_edit = '';
        $labels_edit = '';
        $new_window_edit = '';

        if (Tools::isSubmit('updatelinksmenutop'))
        {
            $link = MenuTopLinks_mod::getLinkLang(Tools::getValue('id_linksmenutop'), (int)Shop::getContextShopID());
            $links_label_edit = $link['link'];
            $labels_edit = $link['label'];
            $new_window_edit = $link['new_window'];
        }			
        
        $fields_values = array(
            'new_window' => Tools::getValue('new_window', $new_window_edit),
            'id_linksmenutop' => Tools::getValue('id_linksmenutop'),
        );

        if (Tools::getValue('submitAddmodule'))
        {
            foreach (Language::getLanguages(false) as $lang)
            {
                $fields_values['label'][$lang['id_lang']] = '';
                $fields_values['link'][$lang['id_lang']] = '';
            }
        }
        else
            foreach (Language::getLanguages(false) as $lang)
            {
                $fields_values['label'][$lang['id_lang']] = Tools::getValue('label_'.(int)$lang['id_lang'], isset($labels_edit[$lang['id_lang']]) ? $labels_edit[$lang['id_lang']] : '');
                $fields_values['link'][$lang['id_lang']] = Tools::getValue('link_'.(int)$lang['id_lang'], isset($links_label_edit[$lang['id_lang']]) ? $links_label_edit[$lang['id_lang']] : '');
            }
        
        return $fields_values;	
    }
    
    public function renderList()
    {
        $links = MenuTopLinks_mod::gets((int)$this->context->language->id, null, (int)Shop::getContextShopID());
        $fields_list = array(
            'id_linksmenutop' => array(
                'title' => $this->l('Link ID'),
                'type' => 'text',
            ),
            'label' => array(
                'title' => $this->l('Label'),
                'type' => 'text',
            ),
            'link' => array(
                'title' => $this->l('Link'),
                'type' => 'link',
            ),
            'new_window' => array(
                'title' => $this->l('New window'),
                'type' => 'bool',
                'align' => 'center',
                'active' => 'status',
            )
        );
        
        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->simple_header = true;
        $helper->identifier = 'id_linksmenutop';
        $helper->table = 'linksmenutop';
        $helper->actions = array('edit', 'delete');
        $helper->show_toolbar = false;
        $helper->module = $this;
        $helper->title = $this->l('Link list');	
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
                
        return $helper->generateList($links, $fields_list);
    }
}